/*
  media-filesystem export utility
  tridge@samba.org, January 2001
  released under the Gnu GPL v2
*/

//
// mfs_stream - An interface rewrite to make it look like ExtractStream
//	for DTivo TivoApp users.
//

#include "mfs.h"

#ifdef TIVO
extern int fixPriority(void); // Force lowest priority keeps DTivo from skipping
#endif


int export_file(int s, u32 fsid)
{
	void	*buf;
	int	bufsize		= 128*1024;
	int	n;
	u64	ofs		= 0;
	u64	size,
		total		= 0;
	int	pct,
		last_pct;

	buf = malloc(bufsize);
	mfs_readahead(1);
	size = mfs_fsid_size(fsid);
	fprintf(stderr, "exporting fsid %d of size %lld to %s\n", fsid, size, "stdout");	fflush(stderr);

	last_pct=0;

	while (total<size &&
	       (n=mfs_fsid_pread(fsid, buf, ofs, MIN(bufsize,size-total))) > 0)
	{
		if (write(s, buf, n) != n)
		{
			fprintf(stderr,"********** #1 failed to write to %s\n", "stdout");	fflush(stderr);
			free(buf);
			return(0);
		}
		ofs += n;
		total += n;
		pct = (100*total)/size;
		if (pct != last_pct)
		{
			fprintf(stderr, "%d%%\r", pct); fflush(stderr);
			last_pct = pct;
		}
	}

	free(buf);     
	return(1);
}


#if !defined(TSERVER)
int verbose = 0;

static void usage(void)
{
	fprintf(stderr, "
usage: mfs_stream [-v] [-s] [-d] [-p] [-h] <fsid>...

options:
        -v Verbose output (stderr)
        -s Dump raw tyStream data to stdout
        -d Dump sector scan info to stderr (NOT SUPPORTED)
        -p Include PES headers with data (does not work, IS NOT SUPPORTED
	-h Help... This text.

Example:
	mfs_stream -s 1 2 3 4 5

NOTE: AUDIO_OUT/VIDEO_OUT env variables not supported. This is an STDOUT raw
tyStream dumper only.

\n");         	fflush(stderr);
	exit(1);
}


int main(int argc, char *argv[])
{
	int fsid;
	int c;
	int mFSID;
	int ready = 0;
	int fd;

#ifdef TIVO
//
// Change our priority at the very start to keep up from hacking the tivo
// software too bad. 
//
	fixPriority();
#endif

	while ((c = getopt(argc, argv, "vhsdp")) != -1 )
	{
		switch (c)
		{
			case 'v':	verbose = 1;	break;
			case 'h':	usage();	break;
			case 's':	ready = 1;	break;
			case 'p':	ready = 0;	break;
			case 'd':	ready = 0;	break;
			default:	usage();	break;
		}
	}

	if (!ready)
	{
		fprintf(stderr, "Incorrect Args...\n");
		usage();
	}

	argc -= optind;
	argv += optind;

	if (argc < 1)	usage();				// Error out.

	fd = fileno(stdout);

	mfs_init();

	for( mFSID = 0; mFSID < argc; mFSID++)
	{
		fsid = mfs_resolve(argv[mFSID]);
		export_file(fd, fsid);
	}

	return(0);
}
#endif

