#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>

#define __need_timeval
#include <time.h>

//#define __need_timeval
//#include <timebits.h>

//
//
// From the MFS_stream code. We are now using this as it seems to work better.
//
#include "mfs.h"
int verbose = 0;
extern int export_file(int s, u32 fsid);

int	SetupClientSocket(char *Servername, int ServerPort);


// DEFINES
//
#define	CMD_SIZE		1024


//
// Globals
//
int	sd;							// Original socket descriptor
int	DelayValue = 50;					// A holder for a simple fix I began trying out.

#ifdef TIVO
	extern int fixPriority(void);				// Force ourselves to a low priority to keep the Tivo from skipping.
#endif


//
// onintr - Our custom interrupt handler, shut down as cleanly as possible.
//
int onintr()							// These routine will be called on a ^C
{
	close(sd);						// Close the socket.
	exit(0);
}


//
// write_error - A handler for the SigPipe interrupt.
//
int write_error()
{
	printf("Attempted to write to a broken socket\n");
	return(0);
}


//
// myAtoi - A custom AtoI function... Saw some wierd behaviour with the standard one.
//
int myAtoi(char *s)
{
	long r = 0;

	while (*s && *s >= '0' && *s <= '9')
	{
		r = 10 * r + *s++ - '0';
	}

	return(r);
}


//
// usage - Print out the usage as seperate lines. Save the deprecation warnings.
//
void usage(void)
{
	printf("tserver_mfs [options]\n");
	printf("Version: 1.0\n");
	printf("options:\n");
	printf("    -n  No Priority Change (Stay at higher priority...)\n");
	printf("    -h  help\n");
	exit(1);
}


//
// main - Where it all begins...
//
main(int argc, char *argv[])
{
	char			buf[CMD_SIZE];			// Command we read from the socket.
	int			ns;				// New socket descriptor
	int			len;
	
	struct sockaddr		client_sa;			// Data structure for the address of the client entity
	struct sockaddr_in	sa_in;				// Structure for address of the server entity.
	long			time_sec;

	extern char		*optarg;
	extern int		optind;
	int			c;
	int			doPri = 1;			// By default change the priority.
	int			reuseAddr;

	while ((c = getopt(argc, argv, "n")) != -1)
	{
		switch (c)
		{
			case 'n':	doPri=0;	break;	// Don't change priority.
			default:	usage();	exit(1);
		}
	}
	
	argc -= optind; argv += optind;
	
	if (argc != 0)
	{
		usage();
	}



#ifdef TIVO
	//
	// Change our priority at the very start to keep up from hacking the tivo software too bad.
	//	
	if (doPri)
	{
		printf("Doing the Lowest PriorityFix...\n");
		fixPriority();				// Force ourselves to a low priority to keep the Tivo from skipping.
	}
	else
	{
		printf("NOT DOING the Lowest PriorityFix...\n");
	}
#endif

	mfs_init();						// Turn on the mfs code so we can read these as we go.


	if (argc == 2)
	{
		DelayValue = atoi(argv[1]);
		fprintf(stderr, "DelayValue = %d\n", DelayValue);
	}

	sd = socket(AF_INET,SOCK_STREAM,0);			// Get a socket descriptor and make sure it is good.
	if (sd<0)
	{
		perror("Can't open a socket\n");
		exit(0);
	}

	bzero(&sa_in, sizeof(sa_in));				// Bind the socket to an address.
	sa_in.sin_addr.s_addr	= INADDR_ANY;			// And legal address in the server interface list.
	sa_in.sin_port		= htons(0xded);			// on port 3565
	sa_in.sin_family	= AF_INET;			// An inet family socket.

	reuseAddr = 1;						// Be able to reopen an already running address.
	setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, (void *)&reuseAddr, sizeof(int));

	if (bind(sd, (struct sockaddr *)&sa_in, sizeof(sa_in)))
	{
		perror("Bad binding in the server");
		close(sd);
		exit(0);
	}
	
	signal(SIGINT,onintr);					// Setup the ^C interrupts and other good ones to watch.
	signal(SIGKILL,onintr);					// All of this so that we shut down as cleanly as possible.
	signal(SIGTERM,onintr);
	signal(SIGPIPE,write_error);
	
	listen(sd, 10);						// Set the listen queue, 10 can wait while we process the current.
	
	for(;;)
	{
		int ret;
		int i;
		
		printf("Waiting for an incomming connection!\n");
		
		ret = waitio (sd, 0);
		if (ret != 1)	continue;			// Something went bad, so leave!
		
		ns = accept(sd,&client_sa,&len);		// Wait here for a client to connect and make a request.
		bzero(buf, CMD_SIZE);
		read(ns, &buf, CMD_SIZE);
		
		for (i=strlen(buf); i>=0; i--)			// Chop the return.
		{
			if (buf[i] == '\0')		buf[i] = '\0';
			else if (buf[i] == '\r')	buf[i] = '\0';
			else if (buf[i] == '\n')	buf[i] = '\0';
			else				break;
		}

		printf("SERVER: We got a message! buf = '%s'\n", buf);
		//
		// CORE: Download a TyStream here...
		//
		if (!strncmp(buf, "TYSTREAM", strlen("TYSTREAM")))	// Do the work here!
		{
			char	seps[]   = "/,";
			char	*token;
			int	FSIDs[100];
			int	reget = -1;
			int	pos = 0;
			int	i;
			int	fsid;

			token = strtok( &(buf[strlen("TYSTREAM")+1]), seps );
			while(token != NULL)			// While there are tokens in "string"
			{
				printf("-> '%s'\n", token);

				FSIDs[pos++] = myAtoi(token);

				fsid = mfs_resolve(token);
				if (!export_file(ns, fsid))	break;		// Stop as soon as we have a problem.

				token = strtok(NULL, seps);
			}

//			printf("pos = %d\n", pos);
//			printf("reget = %d\n", reget);
#ifdef TIVO
//			if (pos)	ForgeMain(ns, pos, FSIDs);				// If we found one, then do it.

/*
			for (i=0; i < pos; i++)
			{
				int fsid = mfs_resolve(FSIDs[i]);
				export_file(ns, fsid);
			}
*/
#endif
		}
		//
		// CORE: Download a NowShowing list here...
		//
		else if (!strncmp(buf, "SHOWING", strlen("SHOWING")))	// Do the work here!
		{
			FILE *fd;
			char buf[256];

			fd = popen("./NowShowing.tcl", "r");
			if (fd)
			{
				while(fgets(buf, 255, fd))
				{
					send(ns, buf, strlen(buf), 0);
				}

				pclose(fd);
			}
			else
			{
				sprintf(buf, "<ERROR>:<Error: Could not find the NowShowing.tcl file...>\r\n");
				send(ns, buf, strlen(buf), 0);
			}
		}
		else if (!strncmp(buf, "DELETE", strlen("DELETE")))	// Do the work here!
		{
			FILE *fd;
			char buf2[256];

			sprintf(buf2, "./NowShowing.tcl %s", buf);

			fd = popen(buf2, "r");
			if (fd)
			{
				while(fgets(buf2, 255, fd))
				{
					char buf3[256];

					sprintf(buf3, "%s\r\n", buf2);
					// Don't do this for now. It causes crashes.
					// send(ns, buf3, strlen(buf3), 0);
				}
				send(ns, "Done.", strlen("Done."), 0);

				pclose(fd);
			}
			else
			{
				sprintf(buf2, "<ERROR>:<Error: Could not find the NowShowing.tcl file...>\r\n");
				send(ns, buf2, strlen(buf2), 0);
			}
		}
		//
		// CORE: Download a TyStream in the new way here... The new way gets us going at full packet sizes.
		//
		else if (!strncmp(buf, "TYSTRM2", strlen("TYSTRM2")))		// Do the work here!
		{
			char	seps[]   = "/,";
			char	*token;
			int	reget = -1;
			int	pos = 0;
			int	i;
			int	fsid;
			int	ns2;
			char	junk1[256];
			char	addr[256];
			char	port[256];

			sscanf(buf, "%s %s %s", junk1, addr, port);
			// printf("Addr = '%s' && port = %d\n", addr, myAtoi(port));

			ns2 = SetupClientSocket(addr, myAtoi(port));
			if (ns2 == -1)
			{
				printf("ERROR: Could not connect to client to send data!\n");
			}
			else
			{
				i = strlen("TYSTRM2")+1 + strlen(addr)+1 + strlen(port)+1;
				token = strtok( &(buf[i]), seps );
				while(token != NULL)			// While there are tokens in "string"
				{
					printf("-> '%s'\n", token);

					fsid = mfs_resolve(token);
					if (!export_file(ns2, fsid))	break;		// Stop as soon as we have a problem.

					token = strtok(NULL, seps);
				}

				close(ns2);
			}
		}
/*
		else if (!strncmp(buf, "TEST", strlen("TEST")))
		{
			char	bigBuf[10 * 4096];
			char	junk1[256];
			char	addr[256];
			char	port[256];
			int	ns2;
			int	i;
			int	fsid;

			for (i=0; i<10 * 4096; i++)
			{
				if (i <     4096)	bigBuf[i] = '1';
				if (i < 2 * 4096)	bigBuf[i] = '2';
				if (i < 3 * 4096)	bigBuf[i] = '3';
				if (i < 4 * 4096)	bigBuf[i] = '4';
				if (i < 5 * 4096)	bigBuf[i] = '5';
				if (i < 6 * 4096)	bigBuf[i] = '6';
				if (i < 7 * 4096)	bigBuf[i] = '7';
				if (i < 8 * 4096)	bigBuf[i] = '8';
				if (i < 9 * 4096)	bigBuf[i] = '9';
				if (i < 10* 4096)	bigBuf[i] = '0';
			}

			sscanf(buf, "%s %s %s", junk1, addr, port);
			printf("Addr = '%s' && port = %d\n", addr, myAtoi(port));

			ns2 = SetupClientSocket(addr, myAtoi(port));
			if (ns2 > -1)
			{
				printf("Socket connected: %d\n", ns2);

				printf("writing: %d\n", 10 * 4096);
//				ret = write(ns2, bigBuf, 10 * 4096);

				// 596245/596248/596251
				fsid = mfs_resolve("596245");
				export_file(ns2, fsid);	

				fsid = mfs_resolve("596248");
				export_file(ns2, fsid);	

				fsid = mfs_resolve("596251");
				export_file(ns2, fsid);	

				printf("And done!\n");

				close(ns2);
			}
		}
*/
		else
		{
			printf("Bogus command... '%s'\n", buf);
		}
		
		close(ns);
	}
}


//
// waitio - A non-busy wait for the socket.
//
int waitio (const int socket, const int secs)
{
	static fd_set	readset;
	struct timeval	TimeDelay;
	int		rslt;
	
	FD_ZERO (&readset);
	FD_SET (socket, &readset);
	
	if (secs)
	{
		TimeDelay.tv_sec  = secs;
		TimeDelay.tv_usec = 0;
		
		rslt = select(socket+1, &readset, NULL, NULL, &TimeDelay);
	}
	else
	{
		rslt = select(socket+1, &readset, NULL, NULL, NULL);
	}

	if (rslt <= 0)	return(rslt);				// We either timed out, or had some other error so leave...
	else if (FD_ISSET(socket, &readset))			// We read something from a socket, just not the right one...
	{
		return(1);					// It worked...
	}
	else
	{
		return(0);					// We read something from a socket, just not the right one...
	}
}


//
// SetupClientSocket - Create and fully connect a client TCP socket. It creats the socket,
//		connects it and returns the new socket.
//
int SetupClientSocket(char *Servername, int ServerPort)
{
	struct sockaddr_in	 sa;
	struct hostent		*hp;
	char			 hostname[256];
	int			 s;

	//
	// Create the socket handle.
	//
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s == -1)
	{
		printf("ERROR: Could not allocate the socket!\n");
		return(-1);
	}

	//
	// Now Create the server address and connect to it.
	//
	if (Servername)	strcpy(hostname, Servername);			// If we have a localname arg, then use it.
	else		gethostname(hostname, 255);			// Otherwise try to get it from the system.

	//
	// Start setting up the address for the socket.
	//
	memset(&sa, 0, sizeof(struct sockaddr_in));
	if (inet_addr(hostname) != -1)					// 1st try to just turn what we were passed into an address.
	{
		sa.sin_addr.s_addr = inet_addr(hostname);
		sa.sin_family = AF_INET;
	}
	else
	{
		//
		// Didn't work, so get the HostEnt by name and see if that worked.
		//
		if ((hp = gethostbyname(hostname)) == NULL)
		{
			printf("ERROR: Host %s not found...\n", hostname);
			return(-1);
		}

		sa.sin_addr.s_addr = *(unsigned long *)hp->h_addr;
		sa.sin_family = hp->h_addrtype;
	}
	sa.sin_port = htons((unsigned short)ServerPort);		// Use the server port arg!

	if (connect(s, (struct sockaddr *)&sa, sizeof(sa)) == -1)
	{
		printf("ERROR: Could not connect the socket!\n");
		return(-1);
	}

	return(s);
}

